@@ -49,6 +49,7 @@ bool_t minus_v = False;
 bool_t override_from = False;
 bool_t rewrite_domain = False;
 bool_t use_tls = False;			/* Use SSL to transfer mail to HUB */
+bool_t use_starttls = False;		/* SSL only after STARTTLS (RFC2487) */
 bool_t use_cert = False;		/* Use a certificate to transfer SSL mail */
 
 #define ARPADATE_LENGTH 32		/* Current date in RFC format */
@@ -106,6 +107,8 @@ void log_event(int priority, char *format, ...)
 	va_end(ap);
 
 #ifdef LOGFILE
+	FILE *fp;
+
 	if((fp = fopen("/tmp/ssmtp.log", "a")) != (FILE *)NULL) {
 		(void)fprintf(fp, "%s\n", buf);
 		(void)fclose(fp);
@@ -126,6 +129,11 @@ void log_event(int priority, char *format, ...)
 #endif
 }
 
+void smtp_write(int fd, char *format, ...);
+int smtp_read(int fd, char *response);
+int smtp_read_all(int fd, char *response);
+int smtp_okay(int fd, char *response);
+
 /*
 dead_letter() -- Save stdin to ~/dead.letter if possible
 */
@@ -883,6 +891,7 @@ bool_t read_config()
 				}
 				else {
 					use_tls = False;
+					use_starttls = False;
 				}
 
 				if(log_level > 0) { 
@@ -890,6 +899,20 @@ bool_t read_config()
 						"Set UseTLS=\"%s\"\n", use_tls ? "True" : "False");
 				}
 			}
+			else if(strcasecmp(p, "UseSTARTTLS") == 0) {
+				if(strcasecmp(q, "YES") == 0) {
+					use_starttls = True;
+					use_tls = True;
+				}
+				else {
+					use_starttls = False;
+				}
+
+				if(log_level > 0) { 
+					log_event(LOG_INFO,
+						"Set UseSTARTTLS=\"%s\"\n", use_tls ? "True" : "False");
+				}
+			}
 			else if(strcasecmp(p, "UseTLSCert") == 0) {
 				if(strcasecmp(q, "YES") == 0) {
 					use_cert = True;
@@ -941,6 +964,7 @@ int smtp_open(char *host, int port)
 
 #ifdef HAVE_SSL
 	int err;
+	char buf[(BUF_SZ + 1)];
 
 	/* Init SSL stuff */
 	SSL_CTX *ctx;
@@ -1040,6 +1064,33 @@ int smtp_open(char *host, int port)
 	if(use_tls == True) {
 		log_event(LOG_INFO, "Creating SSL connection to host");
 
+		if (use_starttls == True)
+		{
+			use_tls=False; /* need to write plain text for a while */
+
+			if (smtp_okay(s, buf))
+			{
+				smtp_write(s, "EHLO %s", hostname);
+				if (smtp_okay(s, buf)) {
+					smtp_write(s, "STARTTLS"); /* assume STARTTLS regardless */
+					if (!smtp_okay(s, buf)) {
+						log_event(LOG_ERR, "STARTTLS not working");
+						return(-1);
+					}
+				}
+				else
+				{
+					log_event(LOG_ERR, "Invalid response: %s (%s)", buf, hostname);
+				}
+			}
+			else
+			{
+				log_event(LOG_ERR, "Invalid response SMTP Server (STARTTLS)");
+				return(-1);
+			}
+			use_tls=True; /* now continue as normal for SSL */
+		}
+
 		ssl = SSL_new(ctx);
 		if(!ssl) {
 			log_event(LOG_ERR, "SSL not working");
@@ -1053,7 +1104,7 @@ int smtp_open(char *host, int port)
 			return(-1);
 		}
 
-		if(log_level > 0) { 
+		if(log_level > 0 || 1) {
 			log_event(LOG_INFO, "SSL connection using %s",
 				SSL_get_cipher(ssl));
 		}
@@ -1249,8 +1300,10 @@ int ssmtp(char *argv[])
 	if((sock = smtp_open(mailhost, port)) == -1) {
 		die("Cannot open %s:%d", mailhost, port);
 	}
-	else if(smtp_okay(sock, buf) == False) {
-		die("Invalid response SMTP server");
+	else if (use_starttls == False) /* no initial response after STARTTLS */
+	{
+		if(smtp_okay(sock, buf) == False)
+			die("Invalid response SMTP server");
 	}
 
 	/* If user supplied username and password, then try ELHO */
@@ -1269,7 +1322,7 @@ int ssmtp(char *argv[])
 	/* Try to log in if username was supplied */
 	if(auth_user) {
 #ifdef MD5AUTH
-		if(auth_pass == (char *)NULL)
+		if(auth_pass == (char *)NULL) {
 			auth_pass = strdup("");
 		}
 
@@ -1283,7 +1336,7 @@ int ssmtp(char *argv[])
 			strncpy(challenge, strchr(buf,' ') + 1, sizeof(challenge));
 
 			memset(buf, 0, sizeof(buf));
-			crammd5(challenge, authUsername, authPassword, buf);
+			crammd5(challenge, auth_user, auth_pass, buf);
 		}
 		else {
 #endif
